from .download import download_zip, download_file
from pathlib import Path
down_path = Path(__file__).parent
xz_url = "addons_file"+"/"+down_path.name+"/"
download_zip(xz_url + "Assets_library.zip",down_path)
download_zip(xz_url + "MeiU_png.zip",down_path)
download_file(xz_url + "__main__.blend",down_path)

import bpy, os, webbrowser
from bpy.types import Panel, Operator, PropertyGroup
from bpy.props import StringProperty, FloatProperty, EnumProperty
from bpy.utils import register_class, unregister_class, previews



bl_info = {
    "blender": (4, 2, 0),
    "version": (2, 8),
    "author": "MeiU",
    "id" : "meiu_addon_hand_drawn_line_generator",
    "license" : "SPDX:GPL-2.0-or-later",
    "maintainer" : "meiu <m@meiu.ary>",
    "location": "View3D > N",
    "doc_url": "https://meiu.art/",
    "name" : "Hand Drawn Line Generator",
    "description": "Easily create hand-drawn lines with convenience",
    "type" : "add-on",
    "category": "Object",
}



# 用於儲存預覽集合的字典
preview_collections = {}

# 这个变量用来存储当前的语言设置，'EN' 代表英文，'zh_HANS' 代表中文
current_language = 'EN'

def toggle_language():
    global current_language
    current_language = 'zh_HANS' if current_language == 'EN' else 'EN'
    # 这里可以添加代码来更新UI上的文本等

class LANGUAGE_OT_toggle(bpy.types.Operator):
    """Toggle Language"""
    bl_idname = "hand_drawn_line_generator_wm.toggle_language"
    bl_label = "Toggle Language"

    def execute(self, context):
        toggle_language()
        # 这里可以添加代码来刷新界面，如果需要的话
        return {'FINISHED'}

# 定義一個字典來存儲所有文本的英文和中文版本
translations = {
    "Adjustment Line": "调整线条",
    "Generate a line": "生成线条",
    "Remove a line": "清除线条",
    "Line style": "线条样式",
    "Line width": "线条宽度",
    "Line color": "线条颜色",
    "Line mark": "线条标记",
    "Mark a line": "标记线条",
    "Unmark a line": "取消标记线条",
    "Mark line": "线条样式",
    "Layer settings": "图层設定",
    "Separate line layers": "分開线条图层",
    "Merge line layers": "合併线条图层",
    "Other features appear only after pressing the generate line button.": "点击生成线条之后才出现其他功能",
}

# 函數用於獲取適當的文本
def get_translated_text(text):
    global current_language
    return text if current_language == 'EN' else translations.get(text, text)

# 讀取資產庫中的資產縮略圖
def load_assets(library_path):
    thumbnails = []
    if os.path.exists(library_path):
        for asset in os.listdir(library_path):
            if asset.endswith('.png'):
                thumbnails.append(asset)
    return thumbnails

class OpenURLOperator(bpy.types.Operator):
    bl_idname = "hand_drawn_line_generator_wm.url_open"
    bl_label = "Open URL"
    url: bpy.props.StringProperty() # type: ignore

    def execute(self, context):
        webbrowser.open(self.url)
        return {'FINISHED'}

def update_asset_previews(self, context):
    """更新資產縮略圖"""
    enum_items = []

    # 檢查是否已經載入過資產縮略圖
    if "main" in preview_collections:
        return preview_collections["main"].enum_items

    # 讀取資產
    library_path = bpy.context.scene.assets_library_path
    thumbnails = load_assets(library_path)

    # 檢查是否需要創建新的預覽集合
    if "main" not in preview_collections:
        pcoll = bpy.utils.previews.new()
        preview_collections["main"] = pcoll
    else:
        pcoll = preview_collections["main"]

    for i, thumbnail in enumerate(thumbnails):
        thumb_path = os.path.join(library_path, thumbnail)
        if thumbnail not in pcoll:
            pcoll.load(thumbnail, thumb_path, 'IMAGE')
        enum_items.append((thumbnail, thumbnail, "", pcoll[thumbnail].icon_id, i))

    pcoll.enum_items = enum_items  # 儲存 enum 項目

    return enum_items

class lineOperator(Operator):
    bl_label = "產生线条"
    bl_idname = "hand_drawn_line_generator_lineno.1"
    button_pressed = False

    def execute(self, context):
        if not lineOperator.button_pressed:
            # 檢查是否已經生成過手繪线条
            
            layout = self.layout
            my_tool = context.scene.my_tool
            
            hand_drawn_line_generator_selected_option = my_tool.hand_drawn_line_generator_my_asset_previews
            
            if hand_drawn_line_generator_selected_option.startswith("手繪线条_") and hand_drawn_line_generator_selected_option.endswith("_1"):
                self.generate_lines_group1()
            # 添加其他手繪线条的判斷條件
            elif hand_drawn_line_generator_selected_option.startswith("手繪线条_") and hand_drawn_line_generator_selected_option.endswith("_2"):
                self.generate_lines_group2()
            elif hand_drawn_line_generator_selected_option.startswith("手繪线条_") and hand_drawn_line_generator_selected_option.endswith("_3"):
                self.generate_lines_group3()
            elif hand_drawn_line_generator_selected_option.startswith("手繪线条_") and hand_drawn_line_generator_selected_option.endswith("_4"):
                self.generate_lines_group4()
                
        return {'FINISHED'}

    def create_new_lineset():
        # 創建新的 Freestyle Lineset
        lineset = bpy.context.scene.render.layers.active.freestyle_settings.linesets.new("New Lineset")

        # 在這裡添加對新Lineset的其他設置，例如线条样式、粗細、顏色等
        lineset.linestyle.name = "MyLineStyle"
        lineset.linestyle.thickness = 2.0
        lineset.linestyle.color = (1.0, 0.0, 0.0, 1.0)  # RGBA顏色

        return lineset

    def generate_lines_group1(self):
        # 生成线条的程式碼_效果1
        
        bpy.context.scene.render.use_freestyle = True

        # 將上下文切換到窗口場景
        override = bpy.context.copy()
        override['window'] = bpy.context.window
        override['scene'] = bpy.context.window.scene

        for view_layer in override['scene'].view_layers:
            view_layer.use_freestyle = True
            try:
                current_lineset = view_layer.freestyle_settings.linesets.active
            except KeyError:
                # 如果出現 KeyError，表示指定的Lineset不存在，可以創建一個新的
                current_lineset = create_new_lineset() # type: ignore

        try:
            bpy.ops.scene.freestyle_lineset_remove()
            # 添加訊息到資訊面板
            self.report({'INFO'}, "Line generated successfully!")
        except Exception as e:
            # 如果出現異常，可以在這裡添加您的處理邏輯
            print(f"無法執行 generate_lines_group1(): {e}")

        bpy.ops.scene.freestyle_lineset_add()
        
        bpy.context.scene.render.line_thickness_mode = 'ABSOLUTE'
        
        # 线条寬度的數值
        bpy.context.scene.render.line_thickness = 1
        
        bpy.context.scene.frame_start = int(2.35323)
        
        # 取得當前的 FreestyleLineSet
        current_lineset = bpy.context.view_layer.freestyle_settings.linesets.active

        # 設置 Edge Type 屬性
        current_lineset.select_edge_mark = True
        
        # 面平滑度
        bpy.context.view_layer.freestyle_settings.use_smoothness = True

        # 皺褶角度的數值
        bpy.context.view_layer.freestyle_settings.crease_angle = 1.9

        bpy.context.area.type = 'VIEW_3D' 

        bpy.context.scene.render.image_settings.compression = 100

        # 檢查 'LineStyle' 是否存在
        if "LineStyle" in bpy.data.linestyles:
            bpy.data.linestyles["LineStyle"].thickness = 10

            # 遍歷所有 linestyles
            for linestyle in bpy.data.linestyles:
                # 檢查 linestyle 名稱是否符合你的條件
                if linestyle.name.startswith("!@$#%%@%^@"):
                    # 添加 ALONG_STROKE alpha_modifier
                    pass
                else:
                    # 如果不符合條件，這裡添加 ALONG_STROKE alpha_modifier
                    linestyle.caps = 'ROUND'
                    alpha_modifier = linestyle.alpha_modifiers.new(name="Along Stroke", type='ALONG_STROKE')
                    alpha_modifier.influence = 0.2

                    # Crease Angle alpha_modifier
                    crease_angle_modifier = linestyle.alpha_modifiers.new(name="Crease Angle", type='CREASE_ANGLE')
                    crease_angle_modifier.influence = 0.2

                    # Along Stroke thickness_modifier
                    thickness_modifier_along_stroke = linestyle.thickness_modifiers.new(name="Along Stroke", type='ALONG_STROKE')
                    thickness_modifier_along_stroke.influence = 0.2

                    # Calligraphy thickness_modifier
                    thickness_modifier_calligraphy = linestyle.thickness_modifiers.new(name="Calligraphy", type='CALLIGRAPHY')
                    thickness_modifier_calligraphy.influence = 0.2

                    # Spatial Noise geometry_modifier
                    geometry_modifier_spatial_noise = linestyle.geometry_modifiers.new(name="Spatial Noise", type='SPATIAL_NOISE')
                    geometry_modifier_spatial_noise.amplitude = 0.2
                    geometry_modifier_spatial_noise.scale = 0.6
        else:
            print("未找到 'LineStyle'")

    def generate_lines_group2(self):
        # 生成线条的程式碼_效果2
        
        bpy.context.scene.render.use_freestyle = True
        
        # 將上下文切換到窗口場景
        override = bpy.context.copy()
        override['window'] = bpy.context.window
        override['scene'] = bpy.context.window.scene

        for view_layer in override['scene'].view_layers:
            view_layer.use_freestyle = True
            try:
                current_lineset = view_layer.freestyle_settings.linesets.active
            except KeyError:
                # 如果出現 KeyError，表示指定的Lineset不存在，可以創建一個新的
                current_lineset = create_new_lineset() # type: ignore

        try:
            bpy.ops.scene.freestyle_lineset_remove()
            # 添加訊息到資訊面板
            self.report({'INFO'}, "Line generated successfully!")
        except Exception as e:
            # 如果出現異常，可以在這裡添加您的處理邏輯
            print(f"無法執行 generate_lines_group1(): {e}")

        bpy.ops.scene.freestyle_lineset_add()
        
        bpy.context.scene.render.line_thickness_mode = 'ABSOLUTE'
        
        # 线条寬度的數值
        bpy.context.scene.render.line_thickness = 2
        
        bpy.context.scene.frame_start = int(2.35323)
        
        # 取得當前的 FreestyleLineSet
        current_lineset = bpy.context.view_layer.freestyle_settings.linesets.active

        # 設置 Edge Type 屬性
        current_lineset.select_edge_mark = True
        
        # 面平滑度
        bpy.context.view_layer.freestyle_settings.use_smoothness = True

        # 皺褶角度的數值
        bpy.context.view_layer.freestyle_settings.crease_angle = 2.0

        bpy.context.scene.render.image_settings.compression = 100
        
        # 檢查 'LineStyle' 是否存在
        if "LineStyle" in bpy.data.linestyles:
            bpy.data.linestyles["LineStyle"].thickness = 10

            # 遍歷所有 linestyles
            for linestyle in bpy.data.linestyles:
                # 檢查 linestyle 名稱是否符合你的條件
                if linestyle.name.startswith("!@$#%%@%^@"):
                    # 添加 ALONG_STROKE alpha_modifier
                    pass
                else:
                    # 如果不符合條件，這裡添加 ALONG_STROKE alpha_modifier
                    linestyle.caps = 'ROUND'

    def generate_lines_group3(self):
        # 生成线条的程式碼_效果3
        
        bpy.context.scene.render.use_freestyle = True
        
        # 將上下文切換到窗口場景
        override = bpy.context.copy()
        override['window'] = bpy.context.window
        override['scene'] = bpy.context.window.scene

        for view_layer in override['scene'].view_layers:
            view_layer.use_freestyle = True
            try:
                current_lineset = view_layer.freestyle_settings.linesets.active
            except KeyError:
                # 如果出現 KeyError，表示指定的Lineset不存在，可以創建一個新的
                current_lineset = create_new_lineset() # type: ignore

        try:
            bpy.ops.scene.freestyle_lineset_remove()
            # 添加訊息到資訊面板
            self.report({'INFO'}, "Line generated successfully!")
        except Exception as e:
            # 如果出現異常，可以在這裡添加您的處理邏輯
            print(f"無法執行 generate_lines_group1(): {e}")
            
        bpy.ops.scene.freestyle_lineset_add()
        
        bpy.context.scene.render.line_thickness_mode = 'ABSOLUTE'
        bpy.context.scene.render.line_thickness = 2.0
        bpy.context.scene.frame_start = int(2.35323)
        bpy.context.area.type = 'VIEW_3D'

        # 取得當前的 FreestyleLineSet
        current_lineset = bpy.context.view_layer.freestyle_settings.linesets.active

        # 設置 Edge Type 屬性
        current_lineset.select_silhouette = False
        current_lineset.select_crease = False
        current_lineset.select_border = False
        current_lineset.select_edge_mark = True
        current_lineset.select_contour = True
        
        # 面平滑度
        bpy.context.view_layer.freestyle_settings.use_smoothness = True

        # 皺褶角度的數值
        bpy.context.view_layer.freestyle_settings.crease_angle = 2.0
        
        bpy.context.scene.render.image_settings.compression = 100
        
        # 檢查 'LineStyle' 是否存在
        if "LineStyle" in bpy.data.linestyles:
            bpy.data.linestyles["LineStyle"].thickness = 10

            # 遍歷所有 linestyles
            for linestyle in bpy.data.linestyles:
                # 檢查 linestyle 名稱是否符合你的條件
                if linestyle.name.startswith("!@$#%%@%^@"):
                    # 添加 ALONG_STROKE alpha_modifier
                    pass
                else:
                    # 如果不符合條件，這裡添加 ALONG_STROKE alpha_modifier
                    linestyle.caps = 'ROUND'

    def generate_lines_group4(self):
        # 生成线条的程式碼_效果4
        
        bpy.context.scene.render.use_freestyle = True
        
        # 將上下文切換到窗口場景
        override = bpy.context.copy()
        override['window'] = bpy.context.window
        override['scene'] = bpy.context.window.scene

        for view_layer in override['scene'].view_layers:
            view_layer.use_freestyle = True
            try:
                current_lineset = view_layer.freestyle_settings.linesets.active
            except KeyError:
                # 如果出現 KeyError，表示指定的Lineset不存在，可以創建一個新的
                current_lineset = create_new_lineset() # type: ignore

        try:
            bpy.ops.scene.freestyle_lineset_remove()
            # 添加訊息到資訊面板
            self.report({'INFO'}, "Line generated successfully!")
        except Exception as e:
            # 如果出現異常，可以在這裡添加您的處理邏輯
            print(f"無法執行 generate_lines_group1(): {e}")

        bpy.ops.scene.freestyle_lineset_add()
        
        bpy.context.scene.render.line_thickness_mode = 'ABSOLUTE'
        
        # 线条寬度的數值
        bpy.context.scene.render.line_thickness = 4
        
        bpy.context.scene.frame_start = int(2.35323)
        
        # 取得當前的 FreestyleLineSet
        current_lineset = bpy.context.view_layer.freestyle_settings.linesets.active

        # 設置 Edge Type 屬性
        current_lineset.select_edge_mark = True
        
        # 面平滑度
        bpy.context.view_layer.freestyle_settings.use_smoothness = True

        # 皺褶角度的數值
        bpy.context.view_layer.freestyle_settings.crease_angle = 1.9

        bpy.context.area.type = 'VIEW_3D' 

        bpy.context.scene.render.image_settings.compression = 100

        # 檢查 'LineStyle' 是否存在
        if "LineStyle" in bpy.data.linestyles:
            bpy.data.linestyles["LineStyle"].thickness = 10

            # 遍歷所有 linestyles
            for linestyle in bpy.data.linestyles:
                # 檢查 linestyle 名稱是否符合你的條件
                if linestyle.name.startswith("!@$#%%@%^@"):
                    # 添加 ALONG_STROKE alpha_modifier
                    pass
                else:
                    # 如果不符合條件，這裡添加 ALONG_STROKE alpha_modifier
                    linestyle.caps = 'SQUARE'

                    alpha_modifier = linestyle.alpha_modifiers.new(name="Along Stroke", type='ALONG_STROKE')
                    alpha_modifier.influence = 0.2

                    # Crease Angle alpha_modifier
                    crease_angle_modifier = linestyle.alpha_modifiers.new(name="Crease Angle", type='CREASE_ANGLE')
                    crease_angle_modifier.influence = 0.2

                    # Along Stroke thickness_modifier
                    thickness_modifier_along_stroke = linestyle.thickness_modifiers.new(name="Along Stroke", type='ALONG_STROKE')
                    thickness_modifier_along_stroke.influence = 0.2

                    # Calligraphy thickness_modifier
                    thickness_modifier_calligraphy = linestyle.thickness_modifiers.new(name="Calligraphy", type='CALLIGRAPHY')
                    thickness_modifier_calligraphy.influence = 0.2

                    # Spatial Noise geometry_modifier
                    geometry_modifier_spatial_noise = linestyle.geometry_modifiers.new(name="Spatial Noise", type='SPATIAL_NOISE')
                    geometry_modifier_spatial_noise.amplitude = 0.2
                    geometry_modifier_spatial_noise.scale = 0.6
                    
                # 添加和设置 Tip Remover 修改器
                tip_remover_found = False
                for modifier in linestyle.geometry_modifiers:
                    if modifier.name == "Tip Remover":
                        tip_remover = modifier
                        tip_remover_found = True
                        break

                if not tip_remover_found:
                    tip_remover = linestyle.geometry_modifiers.new(name="Tip Remover", type='TIP_REMOVER')

                tip_remover.tip_length = 12

                # 添加和设置 Simplification 修改器
                simplification_found = False
                for modifier in linestyle.geometry_modifiers:
                    if modifier.name == "Simplification":
                        simplification = modifier
                        simplification_found = True
                        break

                if not simplification_found:
                    simplification = linestyle.geometry_modifiers.new(name="Simplification", type='SIMPLIFICATION')

                simplification.tolerance = 6
        else:
            print("未找到 'LineStyle'")

class MeiUEnumItem(PropertyGroup):
    name: bpy.props.StringProperty() # type: ignore
    description: bpy.props.StringProperty() # type: ignore
    icon: bpy.props.StringProperty() # type: ignore
    value: bpy.props.IntProperty() # type: ignore

class PictureLocation(PropertyGroup):
    pcoll = bpy.utils.previews.new()
    
    # 在插件開頭加入這行，獲取腳本所在的目錄
    addon_dir = os.path.dirname(__file__)
    
    # 資產庫路徑，使用 addon_dir 變數
    pcoll.images_location = os.path.join(addon_dir, "Assets_library")
    
    preview_collections["hand_drawn_line_generator_my_asset_previews"] = pcoll
    pcoll = preview_collections["hand_drawn_line_generator_my_asset_previews"]
    image_location = pcoll.images_location
    
    # 手動指定圖片路徑
    image_paths = [
        ("手繪线条_1", "LineStyle_1"),
        ("手繪线条_2", "LineStyle_2"),
        ("手繪线条_3", "LineStyle_3"),
        ("手繪线条_4", "LineStyle_4"),
    ]
    
    enum_items = []
    
    for i, (image_label, image_filename) in enumerate(image_paths):
        filepath = os.path.join(image_location, f"{image_filename}.png")
        thumb = pcoll.load(filepath, filepath, 'IMAGE')
        enum_items.append((f"手繪线条_{i+1}", image_label, "", thumb.icon_id, i))
    
    hand_drawn_line_generator_my_asset_previews: EnumProperty(
        name="選擇樣式",
        items=enum_items
    ) # type: ignore
    
    # 添加缺少的屬性定義
    my_string: StringProperty(
        name="My String",
        description="A simple string property",
        default=""
    ) # type: ignore

    my_float: FloatProperty(
        name="My Float",
        description="A simple float property",
        default=0.0
    ) # type: ignore

class lineoff(Operator):
    bl_label = "消除线条"
    bl_idname = "hand_drawn_line_generator_lineno.2"

    def execute(self, context):
        # 關閉线条的程式碼
        bpy.context.scene.render.use_freestyle = False
        # 添加訊息到資訊面板
        self.report({'INFO'}, "Lines have been removed.")
        return {'FINISHED'}

class lineon(Operator):
    bl_label = "顯示线条"
    bl_idname = "hand_drawn_line_generator_lineno.3"

    def execute(self, context):
        # 開啟线条的程式碼
        bpy.context.scene.render.use_freestyle = True
        return {'FINISHED'}

class line_mark_on(Operator):
    bl_label = "标记线条"
    bl_idname = "hand_drawn_line_generator_lineno.4"

    def execute(self, context):
        # 检查当前是否有对象被选中且处于编辑模式
        obj = context.active_object
        if not obj or obj.mode != 'EDIT':
            self.report({'WARNING'}, "This operation needs to be performed in Edit Mode. Please switch to Edit Mode.")
            return {'CANCELLED'}
        
        self.report({'INFO'}, "Mark a Line")

        # 若当前处于编辑模式，则继续执行标记线条的代码
        bpy.ops.mesh.mark_freestyle_edge(clear=False)
        
        return {'FINISHED'}

class line_mark_off(Operator):
    bl_label = "取消标记线条"
    bl_idname = "hand_drawn_line_generator_lineno.5"

    def execute(self, context):
        # 检查当前是否有对象被选中且处于编辑模式
        obj = context.active_object
        if not obj or obj.mode != 'EDIT':
            self.report({'WARNING'}, "This operation needs to be performed in Edit Mode. Please switch to Edit Mode.")
            return {'CANCELLED'}

        self.report({'INFO'}, "Unmark a Line")
                
        # 取消标记线条的程式碼
        bpy.ops.mesh.mark_freestyle_edge(clear=True)
        return {'FINISHED'}

class line_layer_on(Operator):
    bl_label = "分開线条图层"
    bl_idname = "hand_drawn_line_generator_lineno.6"

    def execute(self, context):
        self.report({'INFO'}, "The layers have been separated.")
        # 分開线条图层的程式碼
        bpy.context.view_layer.freestyle_settings.as_render_pass = True
        return {'FINISHED'}

class line_layer_off(Operator):
    bl_label = "合併线条图层"
    bl_idname = "hand_drawn_line_generator_lineno.7"

    def execute(self, context):
        self.report({'INFO'}, "The layers have been merged.")
        # 合併线条图层的程式碼
        bpy.context.view_layer.freestyle_settings.as_render_pass = False
        return {'FINISHED'}

script_dir = os.path.dirname(os.path.abspath(__file__))

# 將預覽集合的管理移至一個初始化函數中
def init_icon_previews():
    preview_collection = bpy.utils.previews.new()
    icons_dir = os.path.join(script_dir, "MeiU_png")
    preview_collection.load("meiu_icon", os.path.join(icons_dir, "meiu.png"), 'IMAGE')
    preview_collection.load("discord_icon", os.path.join(icons_dir, "discord.png"), 'IMAGE')
    preview_collection.load("twitterx_icon", os.path.join(icons_dir, "twitterx.png"), 'IMAGE')
    preview_collection.load("Website_icon", os.path.join(icons_dir, "Website.png"), 'IMAGE')
    return preview_collection

class MeiUInfoPanel(bpy.types.Panel):
    bl_label = "MeiU Info"
    bl_idname = "Hand_Drawn_Line_PT_MeiUInfoPanel"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Hand Drawn Line"

    def draw_header(self, context):
        layout = self.layout
        icon = preview_collections["meiu_icon"]
        layout.label(text="", icon_value=icon.icon_id)

    def draw(self, context):
        layout = self.layout
        row = layout.row()
        row.operator("hand_drawn_line_generator_meiu.button1_operator", icon_value=preview_collections["discord_icon"].icon_id)
        row.operator("hand_drawn_line_generator_meiu.button2_operator", icon_value=preview_collections["twitterx_icon"].icon_id)
        row.operator("hand_drawn_line_generator_meiu.button3_operator", icon_value=preview_collections["Website_icon"].icon_id)

        # 先创建一个框架
        box = layout.box()

        # 在框架内创建一行
        row = box.row()
        
        # 切換語言的按鈕
        op = row.operator("hand_drawn_line_generator_wm.toggle_language", text="En < 中" if current_language == 'EN' else "En > 中")
        box.scale_y = 2

class MeiUButton1Operator(bpy.types.Operator):
    bl_idname = "hand_drawn_line_generator_meiu.button1_operator"
    bl_label = "Discord"

    def execute(self, context):
        # 在這裡打開指定的網址
        webbrowser.open("https://discord.gg/jU7Mj5F3bk")
        return {'FINISHED'}

class MeiUButton2Operator(bpy.types.Operator):
    bl_idname = "hand_drawn_line_generator_meiu.button2_operator"
    bl_label = "Twitter X"

    def execute(self, context):
        # 在這裡打開指定的網址
        webbrowser.open("https://twitter.com/MeiU800")
        return {'FINISHED'}

class MeiUButton3Operator(bpy.types.Operator):
    bl_idname = "hand_drawn_line_generator_meiu.button3_operator"
    bl_label = "Website"

    def execute(self, context):
        # 在這裡打開指定的網址
        webbrowser.open("https://meiu.art/")
        return {'FINISHED'}

class ASSETS_PT_library_panel(Panel):
    """在N面板顯示資產庫"""
    bl_label = "Hand Drawn Line Generator"
    bl_idname = "hand_drawn_line_generator_ASSETS_PT_library"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Hand Drawn Line"
    
    def draw_header(self, context):
        # 使用 Blender 內建的圖示 "WORLD_DATA"
        layout = self.layout
        layout.operator("hand_drawn_line_generator_wm.url_open", text="", icon='QUESTION').url = "https://syeon800.notion.site/034abbea8ab848b38337804e88e805c2?pvs=4"

    def draw(self, context):
        layout = self.layout
        scene = context.scene
        my_tool = scene.my_tool
         
        row = layout.row()
        row = layout.row()
        
        row.label(text=get_translated_text("Line style"), icon='NORMALIZE_FCURVES')
        
        row = layout.row()

        # 使用縮略圖顯示
        row.template_icon_view(my_tool, "hand_drawn_line_generator_my_asset_previews", show_labels=False, scale=10, scale_popup=10)

        row = layout.row()
        
        row = layout.row()
        
        row.operator(lineOperator.bl_idname, text=get_translated_text("Generate a line"), icon='RESTRICT_RENDER_OFF')
        
        row = layout.row()
        
        row.operator(lineoff.bl_idname, text=get_translated_text("Remove a line"), icon='RESTRICT_RENDER_ON')

        row = layout.row()
        row = layout.row()
        row = layout.row()
        
        if context.scene.render.use_freestyle:
            # 取得當前的 FreestyleLineSet，使用 get 方法
            current_lineset = bpy.context.view_layer.freestyle_settings.linesets.active

            # 檢查 current_lineset 是否存在
            if current_lineset:
                row = layout.row()
                row.label(text=get_translated_text("Adjustment Line"), icon='IPO_ELASTIC')
                
                row = layout.row()
                row.prop(context.scene.render, "line_thickness", text=get_translated_text("Line width"))
                
                row = layout.row()
                
                # 只顯示當前 linestyle 的顏色
                active_linestyle = bpy.context.view_layer.freestyle_settings.linesets.active.linestyle
                row.prop(active_linestyle, "color", text=get_translated_text("Line color"))
                
                row = layout.row()
                row = layout.row()
                row = layout.row()
                
                row.label(text=get_translated_text("Line mark"), icon='TRACKER')
                
                row = layout.row()
                row.operator(line_mark_on.bl_idname, text=get_translated_text("Mark a line"), icon='PINNED')
                
                row = layout.row()
                row.operator(line_mark_off.bl_idname, text=get_translated_text("Unmark a line"), icon='UNPINNED')
                
                row = layout.row()
                row = layout.row()
                row = layout.row()
                
                row.label(text=get_translated_text("Layer settings"), icon='DOCUMENTS')
                
                row = layout.row()
                row.operator(line_layer_on.bl_idname, text=get_translated_text("Separate line layers"), icon='SELECT_SUBTRACT')
                
                row = layout.row()
                row.operator(line_layer_off.bl_idname, text=get_translated_text("Merge line layers"), icon='SELECT_EXTEND')
                
                row = layout.row()
                
            else:
                row.label(text="未找到 'View Layer'", icon='ERROR')
            
        else:
            row.label(text=get_translated_text("Other features appear only after pressing the generate line button."), icon='ERROR')

classes = [LANGUAGE_OT_toggle, OpenURLOperator, lineOperator, MeiUEnumItem, PictureLocation, lineoff, lineon, 
           line_mark_on, line_mark_off, 
           line_layer_on, line_layer_off, 
           MeiUInfoPanel, MeiUButton1Operator, MeiUButton2Operator,
           MeiUButton3Operator, ASSETS_PT_library_panel]

assets_library_path = os.path.join(os.path.dirname(__file__), "Assets_library")
preview_collections = {}  # 確保這個全局變數在文件頂部被定義

def register():
    # 註冊所有的類
    for cls in classes:
        bpy.utils.register_class(cls)
    
    # 初始化預覽集合
    global preview_collections
    preview_collections = init_icon_previews()

    # 添加自定義屬性到場景
    bpy.types.Scene.assets_library_path = bpy.props.StringProperty(
        name="Assets Library Path", subtype='DIR_PATH', description="Path to the folder containing asset thumbnails", default=assets_library_path)
    bpy.types.Scene.my_tool = bpy.props.PointerProperty(type=PictureLocation)
    bpy.types.Scene.hand_drawn_line_generator_my_asset_previews = bpy.props.EnumProperty(items=update_asset_previews)
    bpy.types.Scene.my_previews = bpy.props.EnumProperty(items=[], options={'HIDDEN'})

def unregister():
    # 解註冊所有類
    for cls in reversed(classes):
        bpy.utils.unregister_class(cls)
    
    # 清除自定義屬性
    del bpy.types.Scene.assets_library_path
    del bpy.types.Scene.my_tool
    del bpy.types.Scene.hand_drawn_line_generator_my_asset_previews
    del bpy.types.Scene.my_previews

    # 清除預覽圖標集合
    global preview_collections
    bpy.utils.previews.remove(preview_collections)
    preview_collections.clear()

if __name__ == "__main__":
    register()
